#!/usr/bin/python3
# -*- coding: utf-8 -*-
# GCompris - ActivityInfo.py
#
# SPDX-FileCopyrightText: 2021 Johnny Jazeix <jazeix@gmail.com>
#
# SPDX-License-Identifier: GPL-3.0-or-later

from PyQt5.QtCore import pyqtProperty, QObject

class ActivityInfo(QObject):
    def __init__(self, parent=None):
        super().__init__(parent)

        # Initialise the value of the properties.
        self._section = ''
        self._credit = ''
        self._manual = ''
        self._prerequisite = ''
        self._author = ''
        self._goal = ''
        self._description = ''
        self._name = ''
        self._title = ''
        self._icon = ''
        self._createdInVersion = 0
        self._difficulty = 0
        self._levels = 0
        self._enabled = True

    @pyqtProperty('QString')
    def section(self):
        return self._section

    @section.setter
    def section(self, section):
        self._section = section

    @pyqtProperty('QString')
    def author(self):
        return self._author

    @author.setter
    def author(self, author):
        self._author = author

    @pyqtProperty('QString')
    def icon(self):
        return self._icon

    @icon.setter
    def icon(self, icon):
        self._icon = icon

    @pyqtProperty('QString')
    def description(self):
        return self._description

    @description.setter
    def description(self, description):
        self._description = description

    @pyqtProperty('QString')
    def name(self):
        return self._name

    @name.setter
    def name(self, name):
        self._name = name

    @pyqtProperty('QString')
    def goal(self):
        return self._goal

    @goal.setter
    def goal(self, goal):
        self._goal = goal

    @pyqtProperty('QString')
    def prerequisite(self):
        return self._prerequisite

    @prerequisite.setter
    def prerequisite(self, prerequisite):
        self._prerequisite = prerequisite

    @pyqtProperty('QString')
    def manual(self):
        return self._manual

    @manual.setter
    def manual(self, manual):
        self._manual = manual

    @pyqtProperty('QString')
    def credit(self):
        return self._credit

    @credit.setter
    def credit(self, credit):
        self._credit = credit

    @pyqtProperty('QString')
    def title(self):
        return self._title

    @title.setter
    def title(self, title):
        self._title = title

    @pyqtProperty(int)
    def difficulty(self):
        return self._difficulty

    @difficulty.setter
    def difficulty(self, difficulty):
        self._difficulty = difficulty

    @pyqtProperty(int)
    def createdInVersion(self):
        return self._createdInVersion

    @createdInVersion.setter
    def createdInVersion(self, createdInVersion):
        self._createdInVersion = createdInVersion

    @pyqtProperty('QString')
    def levels(self):
        return self._levels

    @levels.setter
    def levels(self, levels):
        self._levels = levels

    @pyqtProperty(bool)
    def enabled(self):
        return self._enabled

    @enabled.setter
    def enabled(self, enabled):
        self._enabled = enabled



#!/usr/bin/python3
# -*- coding: utf-8 -*-
# GCompris - ApplicationInfo.py
#
# SPDX-FileCopyrightText: 2021 Johnny Jazeix <jazeix@gmail.com>
#
# SPDX-License-Identifier: GPL-3.0-or-later

from PyQt5.QtCore import pyqtProperty, QObject, pyqtSignal

class ApplicationInfo(QObject):
    box2DInstalledChanged = pyqtSignal()

    def __init__(self, parent=None):
        super().__init__(parent)

        # Initialise the value of the properties.
        self._isBox2DInstalled = True
        self._isMobile = False

    @pyqtProperty(bool, notify=box2DInstalledChanged)
    def isBox2DInstalled(self):
        return self._isBox2DInstalled

    @isBox2DInstalled.setter
    def isBox2DInstalled(self, isBox2DInstalled):
        self._isBox2DInstalled = isBox2DInstalled

    def createSingleton(self, engine):
        return ApplicationInfo(self)



#!/usr/bin/python3
#
# GCompris - datasetToPo.py
#
# SPDX-FileCopyrightText: 2015 Bruno Coudoin <bruno.coudoin@gcompris.net>
#
#   SPDX-License-Identifier: GPL-3.0-or-later

import sys
import json
import os
import datetime
import polib
import urllib
from urllib.parse import quote

if(len(sys.argv) < 3):
    print("Usage: dataSetToPo.py dataset.json output.po [content-fr.json]")
    print("  The optional argument is used to backport manually created json")
    sys.exit(1)

def loadManualFile(manual):
    json_data = open(manual)
    manualData = json.load(json_data)
    json_data.close()
    return manualData
    
def getManualTranslation(manualData, key):
    if not manualData:
        return ""
    key = key.split("/")[-1]
    try:
        return manualData[key]
    except:
        return ""

manualData = None
if(len(sys.argv) == 4):
    manualData = loadManualFile(sys.argv[3])

dataset = sys.argv[1]
json_data = open(dataset)
data = json.load(json_data)
json_data.close()

displayInConsole = False

# Get last modification time of data set
modtime = os.path.getmtime(dataset)
modtime_utc = datetime.datetime.utcfromtimestamp(modtime)
modtime_utc_string = modtime_utc.strftime('%Y-%m-%d %H:%M') + '+0000'

# Header
po = polib.POFile()
po.metadata = {
    'Project-Id-Version': 'gcompris_qt\\n',
    'Report-Msgid-Bugs-To': 'https://bugs.kde.org/enter_bug.cgi?product=gcompris',
    'POT-Creation-Date': modtime_utc_string,
    'PO-Revision-Date': modtime_utc_string,
    'Last-Translator': 'FULL NAME <EMAIL@ADDRESS>\n',
    'Language-Team': 'LANGUAGE <kde-i18n-doc@kde.org>\n',
    'MIME-Version': '1.0',
    'Content-Type': 'text/plain; charset=utf-8',
    'Content-Transfer-Encoding': '8bit',
}

for chapter in data:
    for lesson in chapter['content']:
        for word in lesson['content']:
            voice = word['voice'].split('/')[-1].split(".")[0] + ".ogg"
            imageLink = "https://gcompris.net/incoming/lang/words_by_section.html#" + \
                       urllib.parse.quote(word['description'].split('/')[-1].split(".")[0])
            if displayInConsole:
                print("#. " + chapter['name'] + \
                      " / " + lesson['name'] + \
                      " / " + word['description'] + \
                      ": "+ imageLink)
                print('msgctxt "'+voice+'"|"')
                print('msgid "' + word['description'] + '"')
                print('msgstr "' + getManualTranslation(manualData, voice) + '"')
                print("")

            entry = polib.POEntry(
                msgid = word['description'],
                msgstr = getManualTranslation(manualData, voice),
                msgctxt = voice,
                comment = chapter['name'] + " / " + lesson['name'] + " / " + word['description'] +
                          "\n" + imageLink
            )
            po.append(entry)

po.save(sys.argv[2])



#!/usr/bin/python3
#
# GCompris - download-assets.py
#
# SPDX-FileCopyrightText: 2016-2018 Johnny Jazeix <jazeix@gmail.com>
#
#   SPDX-License-Identifier: GPL-3.0-or-later
import urllib.request
import shutil
import sys
import os

# see https://stackoverflow.com/questions/7243750/download-file-from-web-in-python-3/7244263#7244263
def download_file(from_url, output_dir):
    file_name = output_dir + "/" + from_url.split('/')[-1]
    print(f"Start to download {file_name}")
    with urllib.request.urlopen(from_url) as response, open(file_name, 'wb') as out_file:
        shutil.copyfileobj(response, out_file)
    print(f"Downloaded {file_name}")

def download_contents_and_file(subfolder, key, downloadContents=True):
    out = OUTPUT_FOLDER + subfolder
    os.makedirs(out, exist_ok=True)
    CONTENTS_FILE = out+"Contents"
    if downloadContents:
        download_file(DOWNLOAD_PATH+subfolder+"Contents", out)
    rcc_line = ""
    with open(CONTENTS_FILE) as f:
        lines = f.readlines()
        for line in lines:
            if key in line:
                rcc_line = line.split()[1]
                break
    download_file(DOWNLOAD_PATH+subfolder+rcc_line, out)


# argv[0]: program name
# argv[1]: assets to download (words, full for full rccs, music, locale to get corresponding locale voices)
# argv[2]: audio format (ogg, mp3, aac)
# argv[3]: output directory (rcc directory)
if len(sys.argv) != 4:
    print("Usage: download-assets.py \"words,full,music,en,fr,pt_BR\" ogg/mp3/aac outputFolder")
    sys.exit(0)

"""Download the voices and words assets depending on the wanted audio format
"""

DOWNLOAD_PATH = "https://cdn.kde.org/gcompris/data3/"
AUDIO_FORMAT = sys.argv[2]
OUTPUT_FOLDER = sys.argv[3]+"/data3/"

ALL_ASSETS = [x.strip() for x in sys.argv[1].split(",") if len(x)]
DOWNLOAD_WORDS = "words" in ALL_ASSETS
DOWNLOAD_FULL = "full" in ALL_ASSETS
DOWNLOAD_MUSIC = "music" in ALL_ASSETS

os.makedirs(OUTPUT_FOLDER, exist_ok=True)

if DOWNLOAD_WORDS:
    ALL_ASSETS.remove("words")
    download_contents_and_file("words/", "words-webp")

if DOWNLOAD_FULL:
    ALL_ASSETS.remove("full")
    download_contents_and_file("", AUDIO_FORMAT)

if DOWNLOAD_MUSIC:
    ALL_ASSETS.remove("music")
    download_contents_and_file("backgroundMusic/", AUDIO_FORMAT)

if ALL_ASSETS:
    downloadContents = True
    for lang in ALL_ASSETS: # We open, read n times the Contents file, could be optimisable but should not take too much time
        download_contents_and_file("voices-"+AUDIO_FORMAT+"/", "-"+lang+"-", downloadContents)
        downloadContents = False




#!/usr/bin/python3
#
# GCompris - mergePo.py
#
# SPDX-FileCopyrightText: 2017 Johnny Jazeix <jazeix@gmail.com>
#
#   SPDX-License-Identifier: GPL-3.0-or-later

# Using polib.py from https://bitbucket.org/izi/polib/wiki/Home

# Usage : python3 mergePo.py qt.po gtk.po
# Find in the first argument po file the translated lines of the gtk po file
# (if existing) and replace the translation for the po file.
# For those not found, no translation is provided.

# something like: python3 mergePo.py gcompris_fi.po fi.po && sed '/^#|/ d' < gcompris_fi.po > qtgl.po && mv qtgl.po gcompris_fi.po

import sys
import polib

if len(sys.argv) < 3:
    print('Usage : python3 mergePo.py gcompris_qt.po gcompris_gtk.po.'
          ' Output will be in the input file!')
    sys.exit(1)

# load an existing po file
po = polib.pofile(sys.argv[1])
poGtk = polib.pofile(sys.argv[2])

print(po.percent_translated())

# First remove all fuzzy strings
for entry in po.fuzzy_entries():
    entry.msgstr = ''
    if entry.msgid_plural:
        entry.msgstr_plural['0'] = ''
    if entry.msgid_plural and '1' in entry.msgstr_plural:
        entry.msgstr_plural['1'] = ''
    if entry.msgid_plural and '2' in entry.msgstr_plural:
        entry.msgstr_plural['2'] = ''
    entry.flags.remove('fuzzy')
    po.save()

# uncomment to create a new po
#for entry in po:
#    if entry.msgstr != "":
#        entry.msgstr = "";

#print(po.percent_translated())

# Then replace in the output all the good strings
for entry in po:
    if entry.msgstr != "":
        continue

    for entryGtk in poGtk:
        if entry.msgid.encode('utf-8') == entryGtk.msgid.encode('utf-8') and entryGtk.msgstr != "":
            entry.msgstr = entryGtk.msgstr

po.save()

print(po.percent_translated())



#!/usr/bin/python3
#
# GCompris - createLaunchImage.py
#
# SPDX-FileCopyrightText: 2015 Bruno Coudoin <bruno.coudoin@gcompris.net>
#
#   SPDX-License-Identifier: GPL-3.0-or-later

import json
import subprocess
import sys
import os
from shutil import copyfile


if len(sys.argv) < 3:
    print('Usage: createLaunchImage.py Images.xcassets/LaunchImage.launchimage ../../src/activities/menu/resource/background.svg')
    sys.exit(1)

outdir = sys.argv[1]
image_source = sys.argv[2]

# Should be extracted from the source image
source_width = 1052
source_height = 610

try:
    os.makedirs(outdir)
except:
    pass

images = [
    {
      "extent" : "full-screen",
      "idiom" : "iphone",
      "subtype" : "736h",
      "width": 1242, "height": 2208,
      "minimum-system-version" : "8.0",
      "orientation" : "portrait",
      "scale" : "3x"
    },
    {
      "extent" : "full-screen",
      "idiom" : "iphone",
      "subtype" : "736h",
      "width": 2208, "height": 1242,
      "minimum-system-version" : "8.0",
      "orientation" : "landscape",
      "scale" : "3x"
    },
    {
      "extent" : "full-screen",
      "idiom" : "iphone",
      "subtype" : "667h",
      "width": 750, "height": 1334,
      "minimum-system-version" : "8.0",
      "orientation" : "portrait",
      "scale" : "2x"
    },
    {
      "orientation" : "portrait",
      "idiom" : "iphone",
      "width": 640, "height": 960,
      "extent" : "full-screen",
      "minimum-system-version" : "7.0",
      "scale" : "2x"
    },
    {
      "extent" : "full-screen",
      "idiom" : "iphone",
      "subtype" : "retina4",
      "width": 640, "height": 1136,
      "minimum-system-version" : "7.0",
      "orientation" : "portrait",
      "scale" : "2x"
    },
    {
      "orientation" : "portrait",
      "idiom" : "ipad",
      "width": 768, "height": 1024,
      "extent" : "full-screen",
      "minimum-system-version" : "7.0",
      "scale" : "1x"
    },
    {
      "orientation" : "landscape",
      "idiom" : "ipad",
      "width": 1024, "height": 768,
      "extent" : "full-screen",
      "minimum-system-version" : "7.0",
      "scale" : "1x"
    },
    {
      "orientation" : "portrait",
      "idiom" : "ipad",
      "width": 1536, "height": 2048,
      "extent" : "full-screen",
      "minimum-system-version" : "7.0",
      "scale" : "2x"
    },
    {
      "orientation" : "landscape",
      "idiom" : "ipad",
      "width": 2048, "height": 1536,
      "extent" : "full-screen",
      "minimum-system-version" : "7.0",
      "scale" : "2x"
    },
    {
      "orientation" : "portrait",
      "idiom" : "iphone",
      "width": 320, "height": 480,
      "extent" : "full-screen",
      "scale" : "1x"
    },
    {
      "orientation" : "portrait",
      "idiom" : "iphone",
      "width": 640, "height": 960,
      "extent" : "full-screen",
      "scale" : "2x"
    },
    {
      "orientation" : "portrait",
      "idiom" : "iphone",
      "width": 640, "height": 1136,
      "extent" : "full-screen",
      "subtype" : "retina4",
      "scale" : "2x"
    },
    {
      "orientation" : "portrait",
      "idiom" : "ipad",
      "width": 768, "height": 1004,
      "extent" : "to-status-bar",
      "scale" : "1x"
    },
    {
      "orientation" : "portrait",
      "idiom" : "ipad",
      "width": 768, "height": 1024,
      "extent" : "full-screen",
      "scale" : "1x"
    },
    {
      "orientation" : "landscape",
      "idiom" : "ipad",
      "width": 1024, "height": 748,
      "extent" : "to-status-bar",
      "scale" : "1x"
    },
    {
      "orientation" : "landscape",
      "idiom" : "ipad",
      "width": 1024, "height": 768,
      "extent" : "full-screen",
      "scale" : "1x"
    },
    {
      "orientation" : "portrait",
      "idiom" : "ipad",
      "width": 1536, "height": 2008,
      "extent" : "to-status-bar",
      "scale" : "2x"
    },
    {
      "orientation" : "portrait",
      "idiom" : "ipad",
      "width": 1536, "height": 2048,
      "extent" : "full-screen",
      "scale" : "2x"
    },
    {
      "orientation" : "landscape",
      "idiom" : "ipad",
      "width": 2048, "height": 1496,
      "extent" : "to-status-bar",
      "scale" : "2x"
    },
    {
      "orientation" : "landscape",
      "idiom" : "ipad",
      "width": 2048, "height": 1536,
      "extent" : "full-screen",
      "scale" : "2x"
    }
]

content = {
    "images": [],
    "info": {
        "version": 1,
        "author": "GCompris"
    }
}

for image in images:
    print(image)
    width = int(image['width'])
    height = int(image['height'])
    sizestr = str(width) + 'x' + str(height)
    image['filename'] = "appicon-" + sizestr + ".png"
    content['images'].append(image)

    # Calc the cropping area
    rw = source_width / width
    rh = source_height / height
    r = min(rw, rh)
    width_final = int(width * r)
    height_final = int(height * r)

    area_x = int((source_width - width_final) / 2)
    area_y = int((source_height - height_final) / 2)
    area = str(area_x) + ':' + \
           str(area_y) + ':' + \
           str(area_x + width_final) + ':' + \
           str(area_y + height_final)

    subprocess.call(["inkscape", image_source,
                      "-e", outdir + '/' + image['filename'],
                      "-a", area,
                      "-w", str(width),
                      "-h", str(height)])

with open(outdir + '/Contents.json', 'w') as f:
    f.write(json.dumps(content, sort_keys=True,
                       indent=4, separators=(',', ': ')))


# Create the Default-658h.png
copyfile(outdir + '/appicon-640x1136.png', outdir + '/../Default-568h@2x.png')



#!/usr/bin/python3
#
# GCompris - fdroid_update_fastlane_metadata.py
#
# SPDX-FileCopyrightText: 2023 Johnny Jazeix <jazeix@gmail.com>
#
#   SPDX-License-Identifier: GPL-3.0-or-later

import sys
import os
import re
import glob
import subprocess
import xml.etree.ElementTree as ET

from PyQt5.QtCore import QCoreApplication, QUrl
from PyQt5.QtQml import qmlRegisterType, QQmlComponent, QQmlEngine
import polib

from python.ActivityInfo import ActivityInfo

if len(sys.argv) < 2:
    print("Usage: fdroid_update_fastlane_metadata.py <version> [-v]")
    sys.exit(1)

version = sys.argv[1]
verbose = len(sys.argv) >= 3 and sys.argv[2] == "-v"

def generate_files_for_locale(changelog_qml, locale):
    # Generate in fastlane/metadata/android/$locale/
    # short_description.txt
    # full_description.txt
    # title.txt
    # changelogs for the version in parameter
    gcompris_locale = locale.replace("-", "_")

    locale_root_dir = f"fastlane/metadata/android/{locale}"
    if not os.path.isdir(locale_root_dir):
        os.mkdir(locale_root_dir)

    suffix = '[@{http://www.w3.org/XML/1998/namespace}lang="%s"]' % locale if locale != "en-US" else ""

    # Open org.kde.gcompris.appdata.xml to retrieve information
    tree = ET.parse("org.kde.gcompris.appdata.xml")
    appdata = tree.getroot()
    children = appdata.findall(f'name{suffix}')
    if not children:
        suffix = '[@{http://www.w3.org/XML/1998/namespace}lang="%s"]' % locale[0:2]
        children = appdata.findall(f'name{suffix}')

    if not children:
        if verbose:
            print(f"Language {locale} not translated")
        return False

    child = children[0]
    with open(f'{locale_root_dir}/title.txt', 'w', encoding="utf8") as title_file:
        title_file.write(child.text)

    children = appdata.findall(f'summary{suffix}')
    if not children:
        print(f"Language {locale} not translated")
        return False

    child = children[0]
    with open(f'{locale_root_dir}/short_description.txt', 'w', encoding="utf8") as short_description_file:
        short_description_file.write(child.text)

    # Get full description from summary elements
    children = appdata.findall(f'description//*{suffix}')
    with open(f'{locale_root_dir}/full_description.txt', 'w', encoding="utf8") as full_description_file:
        for c in children:
            if locale == "en-US" and c.attrib:
                continue
            if c.tag == 'ul':
                continue
            if c.tag == 'li':
                full_description_file.write("- ")
            full_description_file.write(c.text + '\n')

    # We use polib to translate as QTranslator only handles .qm files and we don't need to have GCompris compiled to generate this file
    translation_file_path = 'poqm/'+gcompris_locale+'/gcompris_qt.po'
    if not os.path.isfile(translation_file_path):
        # Short locale
        gcompris_locale = gcompris_locale[0:gcompris_locale.find("_")]
        translation_file_path = 'poqm/'+gcompris_locale+'/gcompris_qt.po'
        if not os.path.isfile(translation_file_path):
            if verbose:
                print(f"Locale {locale} [{gcompris_locale}] not handled, skip it")
            return False
    if verbose:
        print(f"Generate for locale {locale} [{gcompris_locale}]")
    po = polib.pofile(translation_file_path)

    output = ""

    # Get new activities for this version
    # With regex for each ActivityInfo.qml, no need to use Qt there
    for activityInfo in glob.glob('src/activities/*/ActivityInfo.qml'):
        try:
            if activityInfo.find("template") != -1:
                continue
            with open(activityInfo, encoding="utf8") as f:
                content = f.readlines()
                title = ""
                for line in content:
                    titleMatch = re.match('.*title:.*\"(.*)\"', line)
                    if titleMatch:
                        title = titleMatch.group(1)

                    m = re.match('.*createdInVersion:(.*)', line)
                    if m:
                        activityVersion = m.group(1)
                        if int(activityVersion) == int(version):
                            po_title = po.find(title)
                            if locale == "en-US":
                                translated_title = title
                            elif po_title and po_title.translated():
                                translated_title = po_title.msgstr
                            else: # The translation is not complete, we skip the language
                                if verbose:
                                    print(f"Skip {locale} because {title} is not translated")
                                return False
                            output += "- " + translated_title + '\n'
        except IOError as e:
            if verbose:
                print(f"ERROR: Failed to parse {activityInfo}: {e.strerror}")

    # Get changelog information
    versions_data = changelog_qml.property('changelog')
    for version_data in versions_data.toVariant():
        if version == str(version_data['versionCode']):
            content = version_data['content']
            for feature in content:
                po_feature = po.find(feature)
                if locale == "en-US":
                    translated_feature = feature
                elif po_feature and po_feature.translated():
                    translated_feature = po.find(feature).msgstr
                else: # The translation is not complete, we skip the language
                    if verbose:
                        print("Skip %s because %s is not translated" % (locale, feature))
                    return False, ""

                output += "- " + translated_feature + '\n'
    changelog_dir = "%s/changelogs" % locale_root_dir
    if not os.path.isdir(changelog_dir):
        os.mkdir(changelog_dir)
    with open(changelog_dir+'/%s.txt' % version, 'w', encoding="utf8") as changelog_file:
        changelog_file.write(output)

    return True

def main(argv):
    if sys.argv[0] != "./tools/fdroid_update_fastlane_metadata.py":
        print("Needs to be run from top level of GCompris")
        sys.exit(1)
    app = QCoreApplication(sys.argv)

    engine = QQmlEngine()
    # We need to register at least one file for GCompris package else it is not found
    qmlRegisterType(ActivityInfo, "GCompris", 1, 0, "ActivityInfo")
    component = QQmlComponent(engine)
    component.loadUrl(QUrl("src/core/ChangeLog.qml"))
    changelog_qml = component.create()

    # List taken from the android list
    for locale in ["en-US", "ar-AR", "az-AZ", "be", "ca", "cs-CZ", "de-DE", "el-GR", "en-GB", "es-ES", "eu-ES", "fi-FI", "fr-FR", "gl-ES", "he", "hr", "hu-HU", "id", "it-IT", "lt", "mk-MK", "ml-IN", "nl-NL", "no-NO", "pl-PL", "pt-BR", "pt-PT", "ro", "ru-RU", "sk", "sl", "sq", "sv-SE", "tr-TR", "uk", "zh-CN", "zh-TW"]:
        is_translation_ok = generate_files_for_locale(changelog_qml, locale)
        if not is_translation_ok and verbose:
            print(f"Error when generating files for {locale}")

    PIPE = subprocess.PIPE
    process = subprocess.Popen(["git", "ls-files", "-o", "-m", "fastlane"], stdout = PIPE, stderr = PIPE)
    stdoutput, _ = process.communicate()
    if stdoutput:
        if verbose:
            print("git ls-files output:\n", stdoutput.decode("utf-8"))
        process = subprocess.Popen(["git", "add", "fastlane"], stdout = PIPE, stderr = PIPE)
        stdoutput, _ = process.communicate()
        process = subprocess.Popen(["git", "commit", "-m", 'fastlane, update metadata'], stdout = PIPE, stderr = PIPE)
        stdoutput, _ = process.communicate()

if __name__ == '__main__':
    main(sys.argv)



#!/usr/bin/python3
#
# GCompris - android_format_changelog.py
#
# SPDX-FileCopyrightText: 2023 Johnny Jazeix <jazeix@gmail.com>
#
#   SPDX-License-Identifier: GPL-3.0-or-later

import sys
import os
import re
import glob

import polib
from PyQt5.QtCore import QCoreApplication, QUrl
from PyQt5.QtQml import qmlRegisterType, QQmlComponent, QQmlEngine
from python.ActivityInfo import ActivityInfo

if len(sys.argv) < 2:
    print("Usage: android_format_changelog.py <version> [-v]")
    sys.exit(1)

version = sys.argv[1]
verbose = len(sys.argv) >= 3 and sys.argv[2] == "-v"

def generate_for_locale(changelog_qml, locale):
    gcompris_locale = locale.replace("-", "_")

    if gcompris_locale == "iw_IL": # android still uses the old iw iso code for Hebrew in their file
        gcompris_locale = "he_IL"
    # We use polib to translate as QTranslator only handles .qm files and we don't need to have GCompris compiled to generate this file
    translation_file_path = 'poqm/'+gcompris_locale+'/gcompris_qt.po'
    if not os.path.isfile(translation_file_path):
        # Short locale
        gcompris_locale = gcompris_locale[0:gcompris_locale.find("_")]
        translation_file_path = 'poqm/'+gcompris_locale+'/gcompris_qt.po'
        if not os.path.isfile(translation_file_path):
            if verbose:
                print("Locale %s [%s] not handled, skip it" % (locale, gcompris_locale))
            return False, ""
    if verbose:
        print("Generate for locale %s [%s]" % (locale, gcompris_locale))
    po = polib.pofile(translation_file_path)

    output = "<%s>\n" % locale

    # Get new activities for this version
    # With regex for each ActivityInfo.qml, no need to use Qt there
    for activityInfo in glob.glob('src/activities/*/ActivityInfo.qml'):
        try:
            if activityInfo.find("template") != -1:
                continue
            with open(activityInfo) as f:
                content = f.readlines()
                title = ""
                for line in content:
                    titleMatch = re.match('.*title:.*\"(.*)\"', line)
                    if titleMatch:
                        title = titleMatch.group(1)

                    m = re.match('.*createdInVersion:(.*)', line)
                    if m:
                        activityVersion = m.group(1)
                        if int(activityVersion) == int(version):
                            po_title = po.find(title)
                            if locale == "en-US":
                                translated_title = title
                            elif po_title and po_title.translated():
                                translated_title = po_title.msgstr
                            else: # The translation is not complete, we skip the language
                                if verbose:
                                    print("Skip %s because %s is not translated" % (locale, title))
                                return False, ""
                            output += "- " + translated_title + '\n'
        except IOError as e:
            if verbose:
                print(f"ERROR: Failed to parse {activityInfo}: {e.strerror}")

    # Get changelog information
    versions_data = changelog_qml.property('changelog')
    for version_data in versions_data.toVariant():
        if version == str(version_data['versionCode']):
            content = version_data['content']
            for feature in content:
                po_feature = po.find(feature)
                if locale == "en-US":
                    translated_feature = feature
                elif po_feature and po_feature.translated():
                    translated_feature = po.find(feature).msgstr
                else: # The translation is not complete, we skip the language
                    if verbose:
                        print("Skip %s because %s is not translated" % (locale, feature))
                    return False, ""

                output += "- " + translated_feature + '\n'

    output += "</%s>\n" % locale
    return True, output

def main(argv):
    if sys.argv[0] != "./tools/android_format_changelog.py":
        print("Needs to be run from top level of GCompris")
        sys.exit(1)
    app = QCoreApplication(sys.argv)

    engine = QQmlEngine()
    # We need to register at least one file for GCompris package else it is not found
    qmlRegisterType(ActivityInfo, "GCompris", 1, 0, "ActivityInfo")

    # TODO Need to check if we hardcode the list from the xml file or if we get it in GCompris...

    component = QQmlComponent(engine)
    component.loadUrl(QUrl("src/core/ChangeLog.qml"))
    changelog_qml = component.create()

    output = ""
    # List taken from the android list in https://android.googlesource.com/platform/frameworks/base/+/master/core/res/res/values/locale_config.xml
    for locale in ["en-US", "az-AZ", "be", "ca", "cs-CZ", "de-DE", "el-GR", "en-GB", "es-ES", "eu-ES", "fi-FI", "fr-FR", "gl-ES", "hr", "hu-HU", "id", "it-IT", "iw-IL", "lt", "mk-MK", "ml-IN", "nl-NL", "no-NO", "pl-PL", "pt-BR", "pt-PT", "ro", "ru-RU", "sk", "sl", "sq", "sv-SE", "tr-TR", "uk", "zh-CN", "zh-TW"]:
        is_translation_ok, locale_changes = generate_for_locale(changelog_qml, locale)
        if is_translation_ok:
            output += locale_changes

    print(output)

if __name__ == '__main__':
    main(sys.argv)
